#pragma once

#include <chrono>
#include <string>
#include <typeindex>

template <typename Class>
class BaseObject {
public:
    // --- internal typedefs ---

    using HRC = std::chrono::high_resolution_clock;
    using TP = std::chrono::time_point<HRC>;

    // --- data structures ---

    const TP pcid;                // parent class id
    const TP cid;                 // class id
    const std::type_index cindex;

    // --- constructors and deconstructors ---

    BaseObject()
    : pcid(TP::duration::zero()), cid(HRC::now()), cindex(typeid (Class))
    {
    }

    BaseObject(const BaseObject& rhs)
    : pcid(rhs.cid), cid(HRC::now()), cindex(rhs.cindex)
    {
    }

    BaseObject(const BaseObject&& rhs)
    : pcid(std::move(rhs.pcid)), cid(std::move(rhs.cid)), cindex(std::move(rhs.cindex))
    {
    }

    virtual ~BaseObject()
    {
    }

    // --- default operators ---

    BaseObject& operator=(const BaseObject& rhs)
    {
        pcid = rhs.cid;
        cid = HRC::now();
        cindex = rhs.cindex;

        return *this;
    }

    BaseObject& operator=(BaseObject&& rhs)
    {
        pcid = std::move(rhs.pcid);
        cid = std::move(rhs.cid);
        cindex = std::move(rhs.cindex);

        return *this;
    }
};
