#pragma once

/*
 * the QVector is NOT holding a pointer, this is C++11, we have rvalue references
 */

#include <QObject>
#include <QVector>
#include "common/NCTypes.hxx"
#include "filesystem/NCFile.hxx"

class NCFilesystem : public QObject {
    Q_OBJECT

    // interna typedefs
    using ColType = NC::FileData::Type;
    using IdentifyType = NC::Identify::Type;
signals:
    // signals to report every vfs change
    void sigFileStatus(quint32 from_row, quint32 to_row, ColType col) const;
    void sigFileAdd(NCFile& file, quint32 row) const;
    void sigFileReplace(NCFile& file, quint32 row) const;
    void sigFileRemove(quint32 row) const;

public:
    // internal constants
    enum : quint32 {
        Error = NC::Meta::Error,
        HSize = sizeof (quint32) * 2
    };

    // constructors and deconstructors
    NCFilesystem(QObject *parent = 0);
    NCFilesystem(const NCFilesystem& rhs);
    NCFilesystem(const NCFilesystem&& rhs);
    virtual ~NCFilesystem();

    // operators
    NCFilesystem& operator=(const NCFilesystem& rhs);
    NCFilesystem& operator=(const NCFilesystem&& rhs);

    bool operator==(const NCFilesystem& rhs) const;
    bool operator!=(const NCFilesystem& rhs) const;

    // basic and status methodes
    quint32 count() const;
    quint32 size() const;
    bool valid() const;
    void clear();
    QString info() const;

    IdentifyType fileIdentify(const QString& filename) const;
    QString fileIdentifyName(const IdentifyType& type) const;
    QString fileIdentifyName(const QString& filename) const;

    // setters and getters
    quint32 load(const QString& filename);
    quint32 save(const QString& filename) const;

    quint32 fileLoad(const QString& filename);
    quint32 fileAdd(const QString& filename);
    quint32 fileAdd(const NCFile& entry);

    quint32 fileSave(const QString& filename, const QString& name) const;
    quint32 fileSave(const QString& filename, const quint32& index) const;
    quint32 fileSave(const QString& filename, const QString& name, const IdentifyType& type) const;
    quint32 fileSave(const QString& filename, const quint32& index, const IdentifyType& type) const;

    NCFile fileGet(const QString& name) const;
    NCFile fileGet(const quint32& index) const;

    NCFile fileReplace(const NCFile& entry, const QString& name);
    NCFile fileReplace(const NCFile& entry, const quint32& index);
    NCFile fileReplace(const QString& filename, const QString& name);
    NCFile fileReplace(const QString& filename, const quint32& index);

    NCFile fileRemove(const QString& name);
    NCFile fileRemove(const quint32& index);

protected:
    void recalc(QVector<NCFile>& files);

    quint32 vfsLoader(const QString& filename, const bool add = false);
    quint32 pakLoader(const QString& filename, const bool add = false);
    quint32 plainLoader(const QString& filename, const bool add = false);
    quint32 zlibLoader(const QString& filename, const bool add = false);

    quint32 vfsSaver(const QString& filename) const;
    quint32 pakSaver(const NCFile& entry, const QString& filename) const;
    quint32 plainSaver(const NCFile& entry, const QString& filename) const;
    quint32 zlibSaver(const NCFile& entry, const QString& filename) const;

private:
    QVector<NCFile> _files;
};
