/*
 * commands:
 *  - quit/exit : die :D
 *  - ls/list : vfs.info();
 *  - cls/clear : vfs.clear()
 *  - size : vfs.size()
 *  - count : vfs.count()
 *  --------------------------------------------------------------------------
 *  - load <file> : vfs.fileLoad()
 *  - add <file> : vfs.fileAdd()
 *  - remove/delete/rem/del <file/index> : vfs.fileRemove()
 *  - save <filename> : vfs.save()
 *  --------------------------------------------------------------------------
 *  - replace/rep <file/index> <filename> : vfs.fileReplace()
 *  - save <file/index> <filename> : vfs.fileSave()
 */

#include <QStringList>
#include <QTextStream>
#include "filesystem/NCFilesystem.hxx"

QTextStream in(stdin);
QTextStream out(stdout);
QTextStream err(stderr);

int main()
{
    NCFilesystem vfs;
    QString input;
    QStringList list;

    while (true)
    {
        out << "(" << vfs.count() << "|" << vfs.size() << ") > " << flush;
        input = in.readLine();
        list = input.split(" ");

        switch (list.count())
        {
            case 1:
            {
                if (list[0].compare("quit", Qt::CaseInsensitive) == 0 || list[0].compare("exit", Qt::CaseInsensitive) == 0)
                    return 0;

                if (list[0].compare("clear", Qt::CaseInsensitive) == 0 || list[0].compare("cls", Qt::CaseInsensitive) == 0)
                    vfs.clear();

                if (list[0].compare("list", Qt::CaseInsensitive) == 0 || list[0].compare("ls", Qt::CaseInsensitive) == 0)
                    out << "list: " << endl << vfs.info() << endl;

                if (list[0].compare("size", Qt::CaseInsensitive) == 0)
                    out << "size: " << vfs.size() << " bytes" << endl;

                if (list[0].compare("count", Qt::CaseInsensitive) == 0)
                    out << "files: " << vfs.count() << endl;

                break;
            }
            case 2:
            {
                if (list[0].compare("load", Qt::CaseInsensitive) == 0)
                    out << "loading of '" << list[1] << "' " << (vfs.fileLoad(list[1]) != vfs.Error ? "successful" : "failed")
                        << endl;

                if (list[0].compare("add", Qt::CaseInsensitive) == 0)
                    out << "adding of '" << list[1] << "' " << (vfs.fileAdd(list[1]) != vfs.Error ? "successful" : "failed")
                        << endl;

                if (list[0].compare("remove", Qt::CaseInsensitive) == 0 || list[0].compare("delete", Qt::CaseInsensitive) == 0
                    || list[0].compare("rem", Qt::CaseInsensitive) == 0 || list[0].compare("del", Qt::CaseInsensitive) == 0)
                {
                    bool ok = true;
                    quint32 index = list[1].toUInt(&ok);

                    if (ok)
                        out << "removing of file at index " << index << " "
                            << (vfs.fileRemove(index).valid() ? "successful" : "failed") << endl;
                    else
                        out << "removing of file '" << list[1] << "' "
                            << (vfs.fileRemove(list[1]).valid() ? "successful" : "failed") << endl;
                }

                if (list[0].compare("save", Qt::CaseInsensitive) == 0)
                    out << "saving vfs as '" << list[1] << "' " << (vfs.save(list[1]) != vfs.Error ? "successful" : "failed")
                        << endl;

                break;
            }
            case 3:
            {
                /*if (list[0].compare("replace", Qt::CaseInsensitive) == 0 || list[0].compare("rep", Qt::CaseInsensitive) == 0)
                {
                    bool ok = true;
                    quint32 index = list[1].toUInt(&ok);

                    if (ok)
                        out << "replacing file at index " << index << " with file '" << list[2] << "' "
                            << (vfs.fileReplace(list[2], index).valid() ? "successful" : "failed") << endl;
                    else
                        out << "replacing file '" << list[1] << " with file " << list[2] << "' "
                            << (vfs.fileReplace(list[2], list[1]).valid() ? "successful" : "failed") << endl;
                }*/

                if (list[0].compare("save", Qt::CaseInsensitive) == 0)
                {
                    bool ok = true;
                    quint32 index = list[1].toUInt(&ok);

                    if (ok)
                        out << "saving file at index " << index << " as file '" << list[2] << "' "
                            << (vfs.fileSave(list[2], index) != vfs.Error ? "successful" : "failed") << endl;
                    else
                        out << "saving file '" << list[1] << "' as file '" << list[2] << "' "
                            << (vfs.fileSave(list[2], list[1]) != vfs.Error ? "successful" : "failed") << endl;
                }

                break;
            }
            case 4:
                break;
            default:
                err << "error: unknown command '" << input << "'" << endl;
        }
    }

    return 0;
}
